library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity Labx7 is
generic(
	CLOCK_PERIOD : integer := 20;
	TAM_DATA	: integer := 8; 	-- Numero de bits do caractere enviado para o LCD
	NUM_DATA	: integer := 32; 	-- Numero de characteres que serão escritos
	DIV_NUM		: integer := 500;	-- 50Mhz/500 = 100KHz = 10us
	BOOT_TIME	: integer := 2000;  -- 2000/100KHz = 20ms
   REFRESH     : integer := 10000000; -- taxa de atualizaçao do LCD, 10000000*20ns = 0,2s => é atualizado a cada 0,2s
	WR_TIME		: integer := 5;	-- 5*10us = 50us
	CLR_TIME   	: integer := 200;	-- 2ms
	DEBOUNCE    : integer := 2500000  -- filtro ruido de trepidaçao, 2500000*20ns = 50ms
);
port(
	CLOCK_50MHz		: in std_logic;
	KEY				: in std_logic_vector (11 downto 0);
	SW					: in std_logic_vector (3 downto 0);
	
	--LCD
	LCD_D		   	: inout	std_logic_vector(7  downto 0); 	-- LCD data is a bidirectional bus...
	LCD_RS 			: out   std_logic;                		-- LCD register select
	LCD_RW  			: out   std_logic;                		-- LCD Read / nWrite
	LCD_EN  			: out   std_logic;                		-- LCD Enable
	LCD_BACKLIGHT	: out   std_logic;
	
	-- USB DEVICE 
	USB_RXFn			:	in std_logic;
	USB_TXEn			:	in std_logic;
	USB_POWERENn 	: 	in std_logic;
	USB_RD 			: 	out std_logic;
	USB_WR 			: 	out std_logic;
	USB_D 			: 	inout std_logic_vector(7 downto 0)
	);
end Labx7;

architecture behavior of Labx7 is

signal CARACTERE : STD_LOGIC_VECTOR(7 downto 0); -- Caractere A, que será enviado pela USB
signal count : integer range 1 to NUM_DATA := 1;
signal w_reset_50mhz	: std_logic;
signal dado_rd : std_logic_vector(7 downto 0);
signal rd_ready : std_logic;
signal flag_rd 	: std_logic := '0';
signal wr_enable: std_logic := '0';

-- Sinal Teclado
signal key_data: std_logic_vector(3 downto 0);
signal ready_key: std_logic;

--sianis LCD
signal start_lcd	: std_logic;
signal data_dec 	: std_logic_vector(TAM_DATA downto 1);
signal idx 			: integer range 1 to NUM_DATA;

-- Declaraçao do teclado
component teclado_base is
generic(
	DEBOUNCE : integer
);
port(
		clk_i				:	in std_logic;
		push_button_i	:	in std_logic_vector (11 downto 0);
		key_o				: 	out std_logic_vector (3 downto 0);
		ready_o			:	out std_logic
	);
end component;

-- Declaraçao do USB_Device
COMPONENT USB_Device
	GENERIC ( CLOCK_PERIOD : INTEGER := 20 );
	PORT
	(
		clk_i		:	 IN STD_LOGIC;
		rst_i		:	 IN STD_LOGIC;
		wr_i		:	 IN STD_LOGIC;
		dado_wr_i		:	 IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		dado_rd_o		:	 OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		rd_ready_o		:	 OUT STD_LOGIC;
		wr_ready_o		:	 OUT STD_LOGIC;
		USB_RXFn		:	 IN STD_LOGIC;
		USB_TXEn		:	 IN STD_LOGIC;
		USB_POWERENn		:	 IN STD_LOGIC;
		USB_RD		:	 OUT STD_LOGIC;
		USB_WR		:	 OUT STD_LOGIC;
		USB_D		:	 INOUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
END COMPONENT;

-- Declaraçao do LCD
COMPONENT lcd_top
	GENERIC ( TAM_DATA : INTEGER; NUM_DATA : INTEGER; DIV_NUM : INTEGER; BOOT_TIME : INTEGER;
		 WR_TIME : INTEGER; CLR_TIME : INTEGER );
	PORT
	(
		CLK_50MHz_i	:	 IN STD_LOGIC;
		rst_i		   :	 IN STD_LOGIC;
		start_i		:	 IN STD_LOGIC;
		data_i		:	 IN STD_LOGIC_VECTOR(tam_data DOWNTO 1);
		idx_o		   :	 OUT INTEGER range 1 to NUM_DATA;
		ready_o		:	 OUT STD_LOGIC;
		LCD_DATA		:	 INOUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		LCD_RS		:	 OUT STD_LOGIC;
		LCD_RW		:	 OUT STD_LOGIC;
		LCD_EN		:	 OUT STD_LOGIC;
		LCD_BLON		:	 OUT STD_LOGIC
	);
END COMPONENT;

-- declaraçao da memoria que armazenara os caracteres a serem escritos no LCD
	type MEM is array (1 to NUM_DATA) of std_logic_vector(TAM_DATA downto 1);
	signal word: MEM := MEM'(
      x"20", -- Nulo
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20",
      x"20"
	);

   
-- Declaração sincronizador de reset
COMPONENT reset_sync
   PORT
	(
		i_clk	               :	 IN STD_LOGIC;
		i_external_reset_n	:	 IN STD_LOGIC;
		o_reset_n      		:	 OUT STD_LOGIC;
		o_reset		         :	 OUT STD_LOGIC
   );
END COMPONENT;   
   
begin

-- rst <= KEY(11);

   --reset <= KEY(11);
	reset_synch_50mhz_inst : reset_sync
	PORT MAP (
		i_clk                => CLOCK_50MHz,
		i_external_reset_n   => KEY(11),
		o_reset		         => open,
		o_reset_n            => w_reset_50mhz
	);


-- Processo para a recepçao dos dados
process(CLOCK_50MHz, w_reset_50mhz)
begin

if w_reset_50mhz = '1' then
	count <= 1;
	flag_rd <= '0';
elsif rising_edge(CLOCK_50MHz) then
	-- Faz a verificaçao se o rd_ready foi de 1 para 0 e depois de 0 para 1,
	-- para saber se há um dado novo na saída do USB_Device.
	if rd_ready = '1' and flag_rd = '1' then
		flag_rd <= '0';
		word(count) <= dado_rd;
		count <= count + 2;
		if count >= NUM_DATA - 1 then
			count <= 1;
		end if;
	elsif rd_ready = '0' then
		flag_rd <= '1';
	end if;
end if;
end process;

teclado:
	teclado_base 
	generic map
	(
	DEBOUNCE => DEBOUNCE
	)
	port map
	(
		clk_i				=> CLOCK_50MHz,
		push_button_i 	=> '0'& KEY(10 downto 0),
		key_o				=> key_data,
		ready_o			=> ready_key
	);

-- Processo para controlar o processo de envio, debounce do botão do teclado
process (CLOCK_50MHz, w_reset_50mhz)
begin
if w_reset_50mhz /= '1' and rising_edge(CLOCK_50MHz) then
		wr_enable <= '0';
		if  key_data /= "1111" and ready_key = '1' then
				caractere <= key_data + x"30";
				wr_enable <= '1';
		end if;
end if;
end process;

-- Instanciaçao do USB_Device
inst_USB_Device: USB_Device
	GENERIC MAP ( CLOCK_PERIOD => CLOCK_PERIOD)
	PORT MAP
	(
		clk_i				=> CLOCK_50MHz,
		rst_i				=> w_reset_50mhz,
		wr_i				=> wr_enable,
		dado_wr_i		=> CARACTERE,
		dado_rd_o		=> dado_rd,
		rd_ready_o		=> rd_ready,
		wr_ready_o		=> open,
		USB_RXFn			=> USB_RXFn,
		USB_TXEn			=> USB_TXEn,
		USB_POWERENn	=> USB_POWERENn,
		USB_RD			=> USB_RD,
		USB_WR			=> USB_WR,
		USB_D				=> USB_D
	);

		-- Instanciação e configuraçao do LCD
   
   inst_lcd_top : lcd_top
   generic map(
		TAM_DATA 	=> TAM_DATA,
		NUM_DATA	=> NUM_DATA,
		--LCD
		DIV_NUM		=> DIV_NUM,
		BOOT_TIME	=> BOOT_TIME,
		WR_TIME		=> WR_TIME,
		CLR_TIME	=> CLR_TIME
      )
	port map (
		CLK_50MHz_i=> CLOCK_50Mhz,
		rst_i		=> w_reset_50mhz,
		start_i	=> start_lcd,
		data_i	=> data_dec,
		idx_o		=> idx,
		ready_o	=> open,
		-- LCD
		LCD_DATA	=> LCD_D,
		LCD_RS 	=> LCD_RS,
		LCD_RW  	=> LCD_RW,
		LCD_EN  	=> LCD_EN,
		LCD_BLON => open
	);
   
   LCD_BACKLIGHT <= SW(3);
	
   data_dec <= word(idx);
   
	-- Processo para a atualizaçao dos dados escritos no LCD
   process (CLOCK_50MHz, w_reset_50mhz)
   variable count_2: integer range 1 to REFRESH := 1;
   
   begin
   if (rising_edge(CLOCK_50MHz) and w_reset_50mhz /= '1') then
      start_lcd <= '0';
      count_2 := count_2 + 1;
      if count_2 = (REFRESH - 4) then -- -4, tempo para o sinal ser interpretado sem problemas
         start_lcd <= '1';
      elsif count_2 = (REFRESH) then 
         count_2 := 1;
      end if;
   end if;
   end process;
   
end behavior;